@@ -53,7 +53,7 @@ class MiniLinq(object):
     
         elif isinstance(jvalue, list):
             # Leverage for literal lists of data in the code
-            return [from_jvalue(v) for v in jvalue]
+            return [MiniLinq.from_jvalue(v) for v in jvalue]
 
         elif isinstance(jvalue, dict):
             # Dictionaries are reserved; they must always have exactly
@@ -175,7 +175,7 @@ class Filter(MiniLinq):
         def iterate(env=env, source_result=source_result): # Python closure workaround
             if self.name:
                 for item in source_result:
-                    if self.predicate.eval(env.bind(name, item)):
+                    if self.predicate.eval(env.bind(self.name, item)):
                         yield item
             else:
                 for item in source_result:
@@ -386,7 +386,9 @@ class Emit(MiniLinq):
         elif isinstance(cell, int):
             return str(cell)
         elif isinstance(cell, datetime):
-            return cell 
+            return cell
+        elif cell is None:
+            return ''
 
         # In all other cases, coerce to a list and join with ',' for now
         return ','.join([self.coerce_cell(item) for item in list(cell)])
@@ -395,7 +397,7 @@ class Emit(MiniLinq):
         try:
             return self.coerce_cell_blithely(cell)
         except Exception:
-            logger.error('Error converting value to exportable form: %r' % cell)
+            logger.exception('Error converting value to exportable form: %r' % cell)
             return ''
             
     def coerce_row(self, row):
